#include "StdAfx.h"
#include "..\..\DuiLib\UIlib.h"
#include "main_frame.hpp"
#include "WorkGroupController.h"
#include "WorkGroupView.h"
#include "WorkGroupActorEvent.h"
#include "SendNoteMananger.h"

#define MAX_RETRY_COUNT			1

const int OrgGroupType = 1;
const int WorkGroupType = 2;

WorkGroupController::WorkGroupController(void)
: m_pModel(new WorkGroupModel())
, m_pView(new WorkGroupView(this, m_pModel))
, m_pActorEvent(new WorkGroupActorEvent(this))
{
    m_pMainFrame = NULL;
	ResetRetryCount();
}

WorkGroupController::~WorkGroupController(void)
{
	if (NULL != m_pView)
	{
		delete m_pView;
		m_pView = NULL;
	}

	if (NULL != m_pModel)
	{
		delete m_pModel;
		m_pModel = NULL;
	}

	if (NULL != m_pActorEvent)
	{
		delete m_pActorEvent;
		m_pActorEvent = NULL;
	}
}

void WorkGroupController::InitView(MainFrame* pFrame)
{
	m_pMainFrame = pFrame;
	m_pView->InitView(pFrame);
}

void WorkGroupController::ShowView( ORGVIEWTYPE eViewType )
{
	if (NULL == m_pView || NULL == m_pModel)
		return;

	ResetRetryCount();

	if (m_pView->IsCreated())
	{
		m_pView->Show(eViewType);
	}
	else
	{
		m_pModel->Clear();
		m_pView->Create(eViewType);
	}
}

void WorkGroupController::SetORGGroupMemberList(CString strParentID, CString strID)
{
	if (m_pView != NULL && m_pModel != NULL)
	{
		m_pView->SetORGGroupMemberList(strParentID, strID);
	}
}	

void WorkGroupController::SetRequestSearch(CString strWord, MemberSearchAction eSearchAction)
{
	trid_t trid;
	NE_SEARCH_MEMBER_PARAM searchParam;
	CString strLangCode = GetLanguageStringUnicode();

	searchParam.strSearchText = strWord;
	searchParam.strFunction = GET_CONFIG()->GetFunctionList();
	searchParam.strLangCode = strLangCode;
	searchParam.strDeptId = L"";

	CString strLevel;
	strLevel.Format(_T("%d"), GET_CONFIG()->LoadSearchLevel());
	searchParam.strLevel = strLevel;

	if (g_pMessengerActor->GetWASActor()->RequestNameSearchList(searchParam, &trid))
	{
		m_pActorEvent->AddNameSearchTrid(trid, eSearchAction);
	}
	else
	{
		//ASSERT(FALSE);
	}
}

void WorkGroupController::SetReadOnly(BOOL bStatus)
{
	if (NULL != m_pView) m_pView->SetReadOnly(bStatus);
}

void WorkGroupController::Closeview(BOOL bDeteleAllData)
{
	if (NULL != m_pView) 
	{
		m_pView->Close();
	}
	if (m_pModel != NULL)
	{
		m_pModel->Clear(bDeteleAllData);
	}
}

void WorkGroupController::CloseAddUserDlg()
{
	if (NULL != m_pView) m_pView->CloseAddUserDlg();
}

void WorkGroupController::ResetRetryCount()
{
	m_nOrgListRetryCount = 0;
	m_nWorkListRetryCount = 0;
}

void WorkGroupController::SetViewLanguagePack()
{
	if (NULL != m_pView) m_pView->SetLanguagePack();
}

const CString WorkGroupController::GetMemberName( const CString& strUserID ) const
{
	CString result;
	result.Empty();

	if (NULL != m_pView)
		result = m_pView->GetMemberName(strUserID);
	
	return result;
}

void WorkGroupController::RequestWorkGroup()
{
	IGTWASActor *pWASActor = g_pMessengerActor->GetWASActor();
	trid_t trid;
	if (pWASActor->RequestMemberGroupList(WorkGroupType, g_stNtalkInfo.strCorp, g_stNtalkInfo.strCorp, GetLanguageStringUnicode(), &trid))
	{
		if (NULL != m_pActorEvent) m_pActorEvent->AddMemberGroupListTrid(trid, ADD_WORKGROUP);
	}
	else
	{
		if (m_nWorkListRetryCount++ < MAX_RETRY_COUNT)
			m_pView->GetWorkGroupDlg()->SetTimer(TIMER_ORG_REQUEST_WORKROUP, 3000, NULL);
	}
}

void WorkGroupController::RequestORGGroup(MemberGroupListAction orgAction)
{
	IGTWASActor *pWASActor = g_pMessengerActor->GetWASActor();
	trid_t trid;
	if (pWASActor->RequestMemberGroupList(OrgGroupType, g_stNtalkInfo.strCorp, g_stNtalkInfo.strCorp, GetLanguageStringUnicode(), &trid))
	{
		if (NULL != m_pActorEvent) 
			m_pActorEvent->AddMemberGroupListTrid(trid, orgAction);
	}
	else
	{
		if (m_nOrgListRetryCount++ < MAX_RETRY_COUNT && (m_pMainFrame!=NULL))
		{
			m_pMainFrame->SetTimer(TIMER_ORG_REQUEST_ORGGROUP, 3000);
		}
	}
}

void WorkGroupController::AddORGGroupList(const GT_ORG_GROUP_DATA_LIST& list, MemberGroupListAction orgAction)
{
	if (NULL == m_pModel || NULL == m_pView)
		return;

	m_pModel->ClearGroups();

	for (size_t i = 0; i < list.nSize; i++)
	{
		const GT_ORG_GROUP_DATA &groupInfo = list.pList[i];

		m_pModel->AddOrgGroup(groupInfo);
	}

	m_pView->InitOrgGroup(orgAction);
}

void WorkGroupController::AddWorkGroupList(const GT_ORG_GROUP_DATA_LIST& list)
{
	if (NULL == m_pModel || NULL == m_pView)
		return;

	m_pModel->ClearGroups();

	for (size_t i = 0; i < list.nSize; i++)
	{
		const GT_ORG_GROUP_DATA &groupInfo = list.pList[i];

		m_pModel->AddWorkGroup(groupInfo);
	}

	m_pView->InitWorkGroup();
}

// ޽ â ˻   â  
void WorkGroupController::CreateSearchResultView(CString& strSearchText)
{
	if (strSearchText.IsEmpty())
		return;

	//if (NULL != m_pView) 
	//	m_pView->CreateSearchResult(strSearchText);

	if (NULL != m_pModel) 
		m_pModel->Clear();

	ResetRetryCount();

	trid_t trid;
	NE_SEARCH_MEMBER_PARAM searchParam;
	CString strLangCode = GetLanguageStringUnicode();

	searchParam.strSearchText = strSearchText;
	searchParam.strFunction = GET_CONFIG()->GetFunctionList();
	searchParam.strLangCode = strLangCode;
	searchParam.strDeptId = L"";

	CString strLevel;
	strLevel.Format(_T("%d"), GET_CONFIG()->LoadSearchLevel());
	searchParam.strLevel = strLevel;

	if (g_pMessengerActor->GetWASActor()->RequestNameSearchList(searchParam, &trid))
	{
		m_pActorEvent->AddNameSearchTrid(trid, ORG_SEARCH);
	}
	else
	{
		//ASSERT(FALSE);
	}
}

void WorkGroupController::RequestNameSearchList(NE_SEARCH_MEMBER_PARAM& searchParam)
{
	IGTWASActor *pWASActor = g_pMessengerActor->GetWASActor();

	trid_t trid;

	if (pWASActor->RequestNameSearchList(searchParam, &trid))
	{
		if (NULL != m_pActorEvent) m_pActorEvent->AddNameSearchTrid(trid, DETAIL_ORG_SEARCH);
	}
	else
	{
		//ASSERT(FALSE);
	}
}

void WorkGroupController::RequestAddUserSearchList(LPCWSTR strSearchText)
{
	if (wcslen(strSearchText) <= 0)
		return;

	IGTWASActor *pWASActor = g_pMessengerActor->GetWASActor();
	
	trid_t trid;
	NE_SEARCH_MEMBER_PARAM searchParam;
	CString strLangCode = GetLanguageStringUnicode();

	searchParam.strSearchText = strSearchText;
	searchParam.strFunction = GET_CONFIG()->GetFunctionList();
	searchParam.strLangCode = strLangCode;
	searchParam.strDeptId = L"";

	CString strLevel;
	strLevel.Format(_T("%d"), GET_CONFIG()->LoadSearchLevel());
	searchParam.strLevel = strLevel;

	if (pWASActor->RequestNameSearchList(searchParam, &trid))
	{
		if (NULL != m_pActorEvent) m_pActorEvent->AddNameSearchTrid(trid, ADD_USER_SEARCH);
	}
	else
	{
		//ASSERT(FALSE);
	}
}

void WorkGroupController::AddMemberList(const NE_MEMBER_DATA_LIST &list, MemberListAction action)
{
	if (NULL == m_pModel || NULL == m_pView)
		return;

	m_pModel->ClearMembers();

	for (size_t i = 0; i < list.nSize; i++)
	{
		const NE_MEMBER_DATA &sMemberInfo = list.pList[i];

		if (false == sMemberInfo.strUserId.empty())
		{
			m_pModel->AddMember(sMemberInfo);
		}
	}
	if (action == UPDATE_VIEW_DETAIL_ORG)
	{
		m_pView->InitMemberList(false, DETAIL_ORG_SEARCH);
	}
	else if (action == UPDATE_VIEW_ORG)
	{
		m_pView->InitMemberList(false, ORG_SEARCH);
	}
	
}

//  ˻    Լ
void WorkGroupController::AddNameSearchList(const NE_SEARCH_MEMBER_DATA_LIST &list, MemberSearchAction action)
{
	if (NULL == m_pModel || NULL == m_pView)
		return;

	m_pModel->ClearMembers();

	for (size_t i = 0; i < list.nSize; i++)
	{
		const NE_MEMBER_DATA &sMemberInfo = list.pList[i];

		if (false == sMemberInfo.strUserId.empty())
		{
			m_pModel->AddMember(sMemberInfo);
		}
	}

	m_pModel->SetSearchTotal(list.nSearchTotal);
	m_pModel->SetSearchPage(list.nPage);

	m_pView->InitMemberList(true, action);
}

//  ߰â ˻    Լ
void WorkGroupController::AddUserDlgSearchList( const NE_SEARCH_MEMBER_DATA_LIST &list )
{
	if (NULL == m_pModel || NULL == m_pView)
		return;

	m_pModel->ClearSelectedUserVector();

	for (size_t i = 0; i < list.nSize; i++)
	{
		const NE_MEMBER_DATA &sMemberInfo = list.pList[i];

		m_pModel->AddSelectdUser(sMemberInfo);
	}

	m_pView->InitAddUserList();
}

void WorkGroupController::RequestWorkGroupMemberList(LPCWSTR strParentID, LPCWSTR strID)
{
	trid_t trid;
	IGTWASActor *pWASActor = g_pMessengerActor->GetWASActor();

	if (pWASActor->RequestMemberList(WorkGroupType, strParentID, strID, GetLanguageStringUnicode(), &trid))
	{
		g_pMessengerActor->RequestUregFavoriteFromPOI(REGPOIGROUP_ORGCHARTDLG);

		WorkGroupActorEvent::MemberListData data;
		data.action = UPDATE_VIEW_DETAIL_ORG;
		if (NULL != m_pActorEvent) m_pActorEvent->AddMemberListTrid(trid, data);
	}
	else
	{
		//ASSERT(FALSE);
	}
}

void WorkGroupController::RequestORGGroupMemberList(LPCWSTR strParentID, LPCWSTR strID, MemberListAction action)
{
	trid_t trid;
	IGTWASActor *pWASActor = g_pMessengerActor->GetWASActor();

	if (pWASActor->RequestMemberList(OrgGroupType, strParentID, strID, GetLanguageStringUnicode(), &trid))
	{
		g_pMessengerActor->RequestUregFavoriteFromPOI(REGPOIGROUP_ORGCHARTDLG);

		WorkGroupActorEvent::MemberListData data;
		data.action = action;
		if (NULL != m_pActorEvent) m_pActorEvent->AddMemberListTrid(trid, data);

	}
	else
	{
		//ASSERT(FALSE);
	}
}

void WorkGroupController::UpdateMemberStatus(LPCWSTR strUserID, GTUserStatusType eUserStatus)
{
	if (NULL == m_pModel || NULL == m_pView)
		return;

	m_pModel->UpdateMemberStatus(strUserID, eUserStatus);
	m_pView->UpdateMemberStatus(strUserID);
}

void WorkGroupController::UpdateMemberIPTStatus(LPCWSTR strUserID, NTIPTStatusType eIPTStatus)
{
	if (NULL == m_pModel || NULL == m_pView)
		return;

	m_pModel->UpdateMemberIPTStatus(strUserID, eIPTStatus);
	m_pView->UpdateMemberIPTStatus(strUserID);
}

void WorkGroupController::AddORGGroupFavorites(LPCWSTR strParentID, LPCWSTR strID, int nGroupID)
{
	trid_t trid;
	IGTWASActor *pWASActor = g_pMessengerActor->GetWASActor();

	if (pWASActor->RequestMemberList(OrgGroupType, strParentID, strID, GetLanguageStringUnicode(), &trid))
	{
		WorkGroupActorEvent::MemberListData data;
		data.action = ADD_FAVORITE;
		data.nGroupID = nGroupID;
		if (NULL != m_pActorEvent) m_pActorEvent->AddMemberListTrid(trid, data);
	}
	else
	{
		//ASSERT(FALSE);
	}
}

void WorkGroupController::AddWorkGroupFavorites(LPCWSTR strParentID, LPCWSTR strID, int nGroupID)
{
	trid_t trid;
	IGTWASActor *pWASActor = g_pMessengerActor->GetWASActor();

	if (pWASActor->RequestMemberList(WorkGroupType, strParentID, strID, GetLanguageStringUnicode(), &trid))
	{
		WorkGroupActorEvent::MemberListData data;
		data.action = ADD_FAVORITE;
		data.nGroupID = nGroupID;
		if (NULL != m_pActorEvent) m_pActorEvent->AddMemberListTrid(trid, data);
	}
	else
	{
		//ASSERT(FALSE);
	}

}

void WorkGroupController::AddFavoriteList(std::vector<wstring> &vecBuddyId, int nGroupID)
{
	if (vecBuddyId.empty() == false)
	{
		trid_t trid;
		if (g_pMessengerActor->RequestAddBuddy(nGroupID, vecBuddyId, _T(""), abtAddBuddy, &trid) == false)
		{
			//ASSERT(FALSE);
			NTALK_LOG->Debug(L"[BUDDY] RequestAddBuddy(%d, %d) Failed", nGroupID, vecBuddyId.size());
		}
		else
			NTALK_LOG->Debug(L"[BUDDY] RequestAddBuddy(%d, %d) Succeeded", nGroupID, vecBuddyId.size());

		m_pActorEvent->SetBuddyActionTrid(trid);
	}
}

void WorkGroupController::ReqeustAddBuddy( const NE_MEMBER_DATA_LIST &list, int nGroupID )
{
	using namespace std;

	vector<wstring> vecBuddyId;

	for (size_t i = 0; i < list.nSize; i++)
	{
		const NE_MEMBER_DATA &memberInfo = list.pList[i];

		if (IsMyID(memberInfo.strUserId.c_str()))
			continue;

		vecBuddyId.push_back(memberInfo.strUserId.c_str());
	}

	AddFavoriteList(vecBuddyId, nGroupID);
}

void WorkGroupController::RequestORGGroupMemberListChat(LPCWSTR strParentID, LPCWSTR strID)
{
	trid_t trid;
	IGTWASActor *pWASActor = g_pMessengerActor->GetWASActor();

	if (pWASActor->RequestMemberList(WorkGroupType, strParentID, strID, GetLanguageStringUnicode(), &trid))
	{
		WorkGroupActorEvent::MemberListData data;
		data.action = MAKE_CHAT;
		if (NULL != m_pActorEvent) m_pActorEvent->AddMemberListTrid(trid, data);
	}
	else
	{
		//ASSERT(FALSE);
	}
}

void WorkGroupController::MakeMemberListChat( const NE_MEMBER_DATA_LIST &list )
{
#if 0
	if(list.nSize<1)
		return false;

	CStringArray arrUser;
	vector<CChatUser*> vecUser;
	for(size_t i = 0; i < list.nSize; i++)
	{
		const NE_MEMBER_DATA &sMemberInfo = list.pList[i];
		CChatUser* pChatUser = new CChatUser();
		pChatUser->SetID(sMemberInfo.strUserId.c_str());
		pChatUser->SetName(sMemberInfo.strName.c_str());
		pChatUser->SetNickName(sMemberInfo.strNickName.c_str());
		pChatUser->SetStatus(ustONLINE);
		pChatUser->SetInviteStatus(FALSE);
		vecUser.push_back(pChatUser);

	}

	if (vecUser.empty() == false)
	{
		if (m_pChatDlg)
		{
			m_pChatDlg->SendMessage(WM_SENDUSERLIST, 0, reinterpret_cast<LPARAM>((vector<CChatUser*>*)&m_vecChatUser));
			m_pChatDlg->m_DT.setClipBoardFormat(_clipBoardFormat);
		}
		else
		{
			if (GET_MESSENGER_DLG()->GetSafeHwnd())
			{
				GET_MESSENGER_DLG()->SendMessage(WM_MAKECHATDLG, 0, reinterpret_cast<LPARAM>((vector<CChatUser*>*)&m_vecChatUser));
			}
		}

	}	
#endif
}

void WorkGroupController::RequestORGGroupMemberListMemo(LPCWSTR strParentID, LPCWSTR strID)
{
	trid_t trid;
	IGTWASActor *pWASActor = g_pMessengerActor->GetWASActor();

	if (pWASActor->RequestMemberList(OrgGroupType, strParentID, strID, GetLanguageStringUnicode(), &trid))
	{
		WorkGroupActorEvent::MemberListData data;
		data.action = SEND_MEMO;
		if (NULL != m_pActorEvent) m_pActorEvent->AddMemberListTrid(trid, data);
	}
	else
	{
		//ASSERT(FALSE);
	}
}

void WorkGroupController::RequestWorkGroupMemberListMemo(LPCWSTR strParentID, LPCWSTR strID)
{
	trid_t trid;
	IGTWASActor *pWASActor = g_pMessengerActor->GetWASActor();

	if (pWASActor->RequestMemberList(WorkGroupType, strParentID, strID, GetLanguageStringUnicode(), &trid))
	{
		WorkGroupActorEvent::MemberListData data;
		data.action = SEND_MEMO;
		if (NULL != m_pActorEvent) m_pActorEvent->AddMemberListTrid(trid, data);
	}
	else
	{
		//ASSERT(FALSE);
	}

}

void WorkGroupController::RequestORGGroupMemberListSMS(LPCWSTR strParentID, LPCWSTR strID)
{
	trid_t trid;
	IGTWASActor *pWASActor = g_pMessengerActor->GetWASActor();

	if (pWASActor->RequestMemberList(OrgGroupType, strParentID, strID, GetLanguageStringUnicode(), &trid))
	{
		WorkGroupActorEvent::MemberListData data;
		data.action = SEND_SMS;
		if (NULL != m_pActorEvent) m_pActorEvent->AddMemberListTrid(trid, data);
	}
	else
	{
		//ASSERT(FALSE);
	}
}

void WorkGroupController::RequestWorkGroupMemberListSMS(LPCWSTR strParentID, LPCWSTR strID)
{
	trid_t trid;
	IGTWASActor *pWASActor = g_pMessengerActor->GetWASActor();

	if (pWASActor->RequestMemberList(WorkGroupType, strParentID, strID, GetLanguageStringUnicode(), &trid))
	{
		WorkGroupActorEvent::MemberListData data;
		data.action = SEND_SMS;
		if (NULL != m_pActorEvent) m_pActorEvent->AddMemberListTrid(trid, data);
	}
	else
	{
		//ASSERT(FALSE);
	}
}



void WorkGroupController::SendMemberListMemo( const NE_MEMBER_DATA_LIST& list )
{
	GET_APP()->m_pSendNoteManager->CreateMemberListNote(list);
}

void WorkGroupController::SendMemberListSMS( const NE_MEMBER_DATA_LIST& list )
{
	
	GET_APP()->m_pSMSDialogManager->CreateMemberListSMS(list);
}


void WorkGroupController::ChangeMyStatus(GTUserStatusType eMyStatus)
{
	if (NULL != m_pView) m_pView->ChangeMyStatus(eMyStatus);
}

bool WorkGroupController::IsMyID(CString strID)
{
	if (strID.CompareNoCase(g_pMessengerActor->GetUserID()) == 0)
		return true;

	return false;
}

bool WorkGroupController::OpenAddUserDlg(CWnd *pParent, int nType)
{
	bool result = false;
	
	if (NULL != m_pView)
		result = m_pView->OpenAddUserDlg(pParent, nType);

	if (result)
		ClearSelectedUserVector();

	return result;
}

bool WorkGroupController::OpenAddUserDlg( WindowImplBase *pParent, int nType )
{
	bool result = false;

	if (NULL != m_pView)
		result = m_pView->OpenAddUserDlg(pParent, nType);

	if (result)
		ClearSelectedUserVector();

	return result;
}

void WorkGroupController::ClearSelectedUserVector()
{
	if (NULL != m_pModel) 
		m_pModel->ClearSelectedUserVector();
}

const std::vector<ORG_MEMBER_DATA> WorkGroupController::GetSelectedUserVector() const
{
	std::vector<ORG_MEMBER_DATA> resultSelectedUserVector;

	if (NULL != m_pModel)
		resultSelectedUserVector = m_pModel->GetSelectedUserVector();

	return resultSelectedUserVector;
}

GTUserStatusType WorkGroupController::GetMemberStatus( LPCWSTR strUserID )
{
	GTUserStatusType result = ustOFFLINE;

	if (NULL != m_pModel)
		result = m_pModel->GetMemberStatus(strUserID);

	return result;
}

int WorkGroupController::GetImageIndexFromStatus(GTUserStatusType eMyStatus)
{
	int nImage = 0;

	switch (eMyStatus)
	{
	case ustOFFLINE:					nImage = 0;		break;
	case ustONLINE:						nImage = 1;		break;
	case ustAWAY:
	case ustMEETING:
	case ustAUTOAWAY:					nImage = 2;		break;
	case ustBUSY:						nImage = 3;		break;
	case ustOUTSIDE:					nImage = 4;		break;
	case ustEAT:						nImage = 5;		break;

	case ustOFFLINE  | ustMOBILE:		nImage = 7;		break;
	case ustONLINE	 | ustMOBILE:		nImage = 1;		break;
	case ustAWAY	 | ustMOBILE:
	case ustMEETING  | ustMOBILE:		nImage = 2;		break;
	case ustAUTOAWAY | ustMOBILE:		nImage = 7;		break;
	case ustBUSY	 | ustMOBILE:		nImage = 3;		break;
	case ustOUTSIDE  | ustMOBILE:		nImage = 4;		break;
	case ustEAT		 | ustMOBILE:		nImage = 5;		break;

	default:							nImage = 0;		break;
	}

	return nImage;
}

void WorkGroupController::RefreshButtonStatus()
{
	if (NULL != m_pView) m_pView->RefreshButtonStatus();
}

trid_t WorkGroupController::GetBuddyActionTridFromWorkgroupDlg()
{
	if (NULL != m_pActorEvent)
		return m_pActorEvent->GetBuddyActionTrid();
	
	return 0;
}

CWnd* WorkGroupController::GetWorkGroupDlg()
{
	if (NULL != m_pView)
		return m_pView->GetWorkGroupDlg();

	return NULL;
}
